#  Created byMartin.cz
#  Copyright (c) Martin Strohalm. All rights reserved.

# import main objects
from . interpols import Interpol, LinInterpol, LogInterpol, PowInterpol

from . scale import Scale
from . ordinal import OrdinalScale
from . levels import LevelScale
from . continuous import ContinuousScale, LinScale, LogScale, PowScale
from . gradient import GradientScale, GradientLinScale, GradientLogScale, GradientPowScale
from . converter import Converter
